import { config, Config } from "../config"
import { sendTelegram } from "./telegram"
import { sendDiscord } from "./discord"
import { sendPushover } from "./pushover"

const alertEndpointMap = new Map(config.alertEndpoints.map(({name, ...x}) => [name, x]))

export const notify = (message: string, endpoint: Config["pollEndpoints"][number]) => {
    for (const dest of endpoint.alertEndpoints) {
        const d = alertEndpointMap.get(dest)
        if (d === undefined) 
            console.log(`Could not find any valid alert destination ${dest}`)
        else if (d.type === "discord-webhook") 
            sendDiscord(d.webhookUrl, message)
        else if (d.type === "telegram")
            for (const chatId of d.chatIds) sendTelegram(chatId, d.token, message)
        else if (d.type === "pushover")
            for (const de of d.destinationIds) sendPushover(de, d.token, message)
    }
}